@@ -20,7 +20,6 @@
 #include "searchfolder.h"
 #include "nepomuksearchurltools.h"
 #include "standardqueries.h"
-#include "resourcestat.h"
 
 #include <QtCore/QFile>
 
@@ -58,8 +57,56 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+using namespace Nepomuk2::Vocabulary;
 
 namespace {
+    void addGenericNepomukResourceData( const Nepomuk2::Resource& res, KIO::UDSEntry& uds, bool includeMimeType = true )
+    {
+        //
+        // Add some random values
+        //
+        uds.insert( KIO::UDSEntry::UDS_ACCESS, 0700 );
+        uds.insert( KIO::UDSEntry::UDS_USER, KUser().loginName() );
+        if ( res.hasProperty( NIE::lastModified() ) ) {
+            // remotely stored files
+            uds.insert( KIO::UDSEntry::UDS_MODIFICATION_TIME, res.property( NIE::lastModified() ).toDateTime().toTime_t() );
+        }
+        else {
+            // all nepomuk resources
+            uds.insert( KIO::UDSEntry::UDS_MODIFICATION_TIME, res.property( Soprano::Vocabulary::NAO::lastModified() ).toDateTime().toTime_t() );
+            uds.insert( KIO::UDSEntry::UDS_CREATION_TIME, res.property( Soprano::Vocabulary::NAO::created() ).toDateTime().toTime_t() );
+        }
+
+        if ( res.hasProperty( NIE::contentSize() ) ) {
+            // remotely stored files
+            uds.insert( KIO::UDSEntry::UDS_SIZE, res.property( NIE::contentSize() ).toInt() );
+        }
+
+
+        //
+        // Starting with KDE 4.4 we have the pretty UDS_NEPOMUK_URI which makes
+        // everything much cleaner since kio slaves can decide if the resources can be
+        // annotated or not.
+        //
+        uds.insert( KIO::UDSEntry::UDS_NEPOMUK_URI, KUrl( res.uri() ).url() );
+
+        if ( includeMimeType ) {
+            // Use nice display types like "Person", "Project" and so on
+            Nepomuk2::Types::Class type( res.type() );
+            if (!type.label().isEmpty())
+                uds.insert( KIO::UDSEntry::UDS_DISPLAY_TYPE, type.label() );
+
+            QString icon = res.genericIcon();
+            if( !icon.isEmpty() ) {
+                uds.insert( KIO::UDSEntry::UDS_ICON_NAME, icon );
+            }
+            else {
+                uds.insert( KIO::UDSEntry::UDS_ICON_NAME, QLatin1String("nepomuk") );
+                uds.remove( KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES );
+            }
+        }
+    }
+
     KIO::UDSEntry statSearchFolder( const KUrl& url ) {
         KIO::UDSEntry uds;
         uds.insert( KIO::UDSEntry::UDS_ACCESS, 0700 );
@@ -71,7 +118,8 @@ namespace {
         uds.insert( KIO::UDSEntry::UDS_NAME, Nepomuk2::Query::Query::titleFromQueryUrl( url ) );
         uds.insert( KIO::UDSEntry::UDS_DISPLAY_NAME, Nepomuk2::Query::Query::titleFromQueryUrl( url ) );
         if ( url.hasQueryItem( QLatin1String( "resource" ) ) ) {
-            Nepomuk2::addGenericNepomukResourceData( Nepomuk2::Resource( KUrl( url.queryItemValue( QLatin1String( "resource" ) ) ) ), uds );
+            Nepomuk2::Resource resource( url.queryItemValue( QLatin1String("resource") ) );
+            addGenericNepomukResourceData( resource, uds );
         }
         Nepomuk2::Query::Query query = Nepomuk2::Query::Query::fromQueryUrl( url );
         if ( query.isValid() )
@@ -92,41 +140,27 @@ namespace {
                 url.directory() == QLatin1String("/") );
     }
 
-    // Legacy query URLs look like: nepomuksearch:/?query=xyz&title=foobar
-    // i.e. an empty path and a query, new URLs have their title as path
+    // Legacy URLs - nepomuksearch:/SomeQuery
     bool isLegacyQueryUrl( const KUrl& url ) {
         const QString path = url.path(KUrl::RemoveTrailingSlash);
-        return( url.hasQuery() &&
-                ( path.isEmpty() || path == QLatin1String("/") ) );
+        return !path.isEmpty() && !url.hasQuery();
     }
 
     KUrl convertLegacyQueryUrl( const KUrl& url ) {
-        KUrl newUrl(QLatin1String("nepomuksearch:/") + Nepomuk2::Query::Query::titleFromQueryUrl(url));
-        Nepomuk2::Query::Query query = Nepomuk2::Query::Query::fromQueryUrl(url);
-        if(query.isValid())
-            newUrl.addQueryItem(QLatin1String("encodedquery"), query.toString());
-        else
-            newUrl.addQueryItem(QLatin1String("sparql"), Nepomuk2::Query::Query::sparqlFromQueryUrl(url));
-        return newUrl;
+        return KUrl( QLatin1String("nepomuksearch:/") +
+                     Nepomuk2::Query::Query::titleFromQueryUrl(url) +
+                     QLatin1String("?query=") +
+                     url.path().section( '/', 0, 0, QString::SectionSkipEmpty ) );
     }
 
     Nepomuk2::Query::Query rootQuery() {
-        KConfig config( "kio_nepomuksearchrc" );
-        QString queryStr = config.group( "General" ).readEntry( "Root query", QString() );
-        Nepomuk2::Query::Query query;
-        if ( queryStr.isEmpty() )
-            query = Nepomuk2::lastModifiedFilesQuery();
-        else
-            query = Nepomuk2::Query::Query::fromString( queryStr );
-        query.setLimit( config.group( "General" ).readEntry( "Root query limit", 10 ) );
-        return query;
+        return Nepomuk2::lastModifiedFilesQuery();
     }
-    const int s_historyMax = 10;
 }
 
 
 Nepomuk2::SearchProtocol::SearchProtocol( const QByteArray& poolSocket, const QByteArray& appSocket )
-    : KIO::ForwardingSlaveBase( "nepomuksearch", poolSocket, appSocket )
+    : KIO::SlaveBase( "nepomuksearch", poolSocket, appSocket )
 {
 }
 
@@ -138,39 +172,34 @@ Nepomuk2::SearchProtocol::~SearchProtocol()
 
 bool Nepomuk2::SearchProtocol::ensureNepomukRunning( bool emitError )
 {
-    if ( Nepomuk2::ResourceManager::instance()->init() ) {
-        kDebug() << "Failed to init Nepomuk";
-        if ( emitError )
-            error( KIO::ERR_SLAVE_DEFINED, i18n( "The desktop search service is not activated. Unable to answer queries without it." ) );
-        return false;
-    }
-    else if ( !Nepomuk2::Query::QueryServiceClient::serviceAvailable() ) {
-        kDebug() << "Nepomuk Query service is not running.";
-        if ( emitError )
-            error( KIO::ERR_SLAVE_DEFINED, i18n( "The desktop search query service is not running. Unable to answer queries without it." ) );
-        return false;
-    }
-    else {
+    ResourceManager* rm = ResourceManager::instance();
+    if ( !rm->initialized() ) {
+        if( rm->init() ) {
+            if( emitError )
+                error( KIO::ERR_SLAVE_DEFINED, i18n( "The desktop search service is not activated. "
+                                                      "Unable to answer queries without it." ) );
+            return false;
+        }
         return true;
     }
+
+    return true;
 }
 
 
-void Nepomuk2::SearchProtocol::listDir( const KUrl& url )
+void Nepomuk2::SearchProtocol::listDir( const KUrl& url_ )
 {
-    kDebug() << url;
+    kDebug() << url_;
+
+    KUrl url = url_;
+    if( isLegacyQueryUrl(url_) )
+        url = convertLegacyQueryUrl(url_);
 
     // list the root folder
     if ( isRootUrl( url ) ) {
         listRoot();
     }
 
-    // backwards compatibility with pre-4.6 query URLs
-    else if( isLegacyQueryUrl( url ) ) {
-        redirection( convertLegacyQueryUrl(url) );
-        finished();
-    }
-
     // list the actual query folders
     else if( isQueryFolder( url ) ) {
         if ( !ensureNepomukRunning(false) ) {
@@ -180,46 +209,26 @@ void Nepomuk2::SearchProtocol::listDir( const KUrl& url )
         }
         else {
             SearchFolder folder( url, this );
-            updateQueryUrlHistory( url );
             folder.list();
             listEntry( KIO::UDSEntry(), true );
             finished();
         }
     }
 
-    // listing of query results that are folders
     else {
-        ForwardingSlaveBase::listDir(url);
+        error( KIO::ERR_CANNOT_ENTER_DIRECTORY, url.prettyUrl() );
+        return;
     }
 }
 
 
-void Nepomuk2::SearchProtocol::get( const KUrl& url )
+void Nepomuk2::SearchProtocol::mimetype( const KUrl& url_ )
 {
-    kDebug() << url;
+    kDebug() << url_;
 
-    if ( !ensureNepomukRunning() )
-        return;
-
-    ForwardingSlaveBase::get( url );
-}
-
-
-void Nepomuk2::SearchProtocol::put( const KUrl& url, int permissions, KIO::JobFlags flags )
-{
-    kDebug() << url << permissions << flags;
-
-    if ( !ensureNepomukRunning() )
-        return;
-
-    // this will work only for existing files (ie. overwrite to allow saving of opened files)
-    ForwardingSlaveBase::put( url, permissions, flags );
-}
-
-
-void Nepomuk2::SearchProtocol::mimetype( const KUrl& url )
-{
-    kDebug() << url;
+    KUrl url = url_;
+    if( isLegacyQueryUrl(url_) )
+        url = convertLegacyQueryUrl(url_);
 
     // the root url is always a folder
     if ( isRootUrl( url ) ) {
@@ -236,16 +245,20 @@ void Nepomuk2::SearchProtocol::mimetype( const KUrl& url )
         finished();
     }
 
-    // results are forwarded
     else {
-        ForwardingSlaveBase::mimetype( url );
+        error( KIO::ERR_CANNOT_ENTER_DIRECTORY, url.prettyUrl() );
+        return;
     }
 }
 
 
-void Nepomuk2::SearchProtocol::stat( const KUrl& url )
+void Nepomuk2::SearchProtocol::stat( const KUrl& url_ )
 {
-    kDebug() << url;
+    kDebug() << url_;
+
+    KUrl url = url_;
+    if( isLegacyQueryUrl(url_) )
+        url = convertLegacyQueryUrl(url_);
 
     // the root folder
     if ( isRootUrl( url ) ) {
@@ -272,44 +285,17 @@ void Nepomuk2::SearchProtocol::stat( const KUrl& url )
         finished();
     }
 
-    // results are forwarded
     else {
-        kDebug() << "Stat forward" << url;
-        ForwardingSlaveBase::stat(url);
+        error( KIO::ERR_CANNOT_ENTER_DIRECTORY, url.prettyUrl() );
+        return;
     }
 }
 
 
-void Nepomuk2::SearchProtocol::del(const KUrl& url, bool isFile)
-{
-    ForwardingSlaveBase::del( url, isFile );
-}
-
-
-bool Nepomuk2::SearchProtocol::rewriteUrl( const KUrl& url, KUrl& newURL )
-{
-    // we do it the speedy but slightly umpf way: decode the encoded URI from the filename
-    newURL = Nepomuk2::udsNameToResourceUri( url.fileName() );
-    kDebug() << "URL:" << url << "NEW URL:" << newURL << newURL.protocol() << newURL.path() << newURL.fileName();
-    return !newURL.isEmpty();
-}
-
-
-void Nepomuk2::SearchProtocol::prepareUDSEntry( KIO::UDSEntry& uds, bool listing ) const
-{
-    // do nothing - we do everything in SearchFolder::statResult
-    Q_UNUSED(uds);
-    Q_UNUSED(listing);
-}
-
-
 void Nepomuk2::SearchProtocol::listRoot()
 {
     kDebug();
 
-    // flush
-    listEntry( KIO::UDSEntry(), true );
-
     Query::Query query = rootQuery();
     if ( query.isValid() ) {
         // FIXME: Avoid this useless conversion to searchUrl and back
@@ -322,54 +308,6 @@ void Nepomuk2::SearchProtocol::listRoot()
 }
 
 
-void Nepomuk2::SearchProtocol::updateQueryUrlHistory( const KUrl& url )
-{
-    //
-    // if the url is already in the history update its timestamp
-    // otherwise remove the last item if we reached the max and then
-    // add the url along with its timestamp
-    //
-    KSharedConfigPtr cfg = KSharedConfig::openConfig( "kio_nepomuksearchrc" );
-    KConfigGroup grp = cfg->group( "Last Queries" );
-
-    // read config
-    const int cnt = grp.readEntry( "count", 0 );
-    QList<QPair<KUrl, QDateTime> > entries;
-    for ( int i = 0; i < cnt; ++i ) {
-        KUrl u = grp.readEntry( QString::fromLatin1( "query_%1_url" ).arg( i ), QString() );
-        QDateTime t = grp.readEntry( QString::fromLatin1( "query_%1_timestamp" ).arg( i ), QDateTime() );
-        if ( !u.isEmpty() &&
-             t.isValid() &&
-             u != url ) {
-            int pos = 0;
-            while ( entries.count() > pos &&
-                    entries[pos].second < t ) {
-                ++pos;
-            }
-            entries.insert( pos, qMakePair( u, t ) );
-        }
-    }
-    if ( entries.count() >= s_historyMax ) {
-        entries.removeFirst();
-    }
-    entries.append( qMakePair( url, QDateTime::currentDateTime() ) );
-
-    // write config back
-    grp.deleteGroup();
-    grp = cfg->group( "Last Queries" );
-
-    for ( int i = 0; i < entries.count(); ++i ) {
-        KUrl u = entries[i].first;
-        QDateTime t = entries[i].second;
-        grp.writeEntry( QString::fromLatin1( "query_%1_url" ).arg( i ), u.url() );
-        grp.writeEntry( QString::fromLatin1( "query_%1_timestamp" ).arg( i ), t );
-    }
-    grp.writeEntry( QLatin1String( "count" ), entries.count() );
-
-    cfg->sync();
-}
-
-
 extern "C"
 {
     KDE_EXPORT int kdemain( int argc, char **argv )
@@ -390,34 +328,4 @@ extern "C"
 }
 
 
-#if 0
-void Nepomuk2::SearchProtocol::listUserQueries()
-{
-    UserQueryUrlList userQueries;
-    Q_FOREACH( const KUrl& url, userQueries ) {
-        KIO::UDSEntry uds = statSearchFolder( url );
-        uds.insert( KIO::UDSEntry::UDS_DISPLAY_TYPE, i18n( "Saved Query" ) );
-        listEntry( uds, false );
-    }
-}
-void Nepomuk2::SearchProtocol::listLastQueries()
-{
-    KSharedConfigPtr cfg = KSharedConfig::openConfig( "kio_nepomuksearchrc" );
-    KConfigGroup grp = cfg->group( "Last Queries" );
-
-    // read config
-    const int cnt = grp.readEntry( "count", 0 );
-    QList<QPair<KUrl, QDateTime> > entries;
-    for ( int i = 0; i < cnt; ++i ) {
-        KUrl u = grp.readEntry( QString::fromLatin1( "query_%1_url" ).arg( i ), QString() );
-        QDateTime t = grp.readEntry( QString::fromLatin1( "query_%1_timestamp" ).arg( i ), QDateTime() );
-        if ( !u.isEmpty() && t.isValid() )
-            listEntry( statLastQuery( u, t ), false );
-    }
-
-    listEntry( KIO::UDSEntry(), true );
-    finished();
-}
-#endif
-
 #include "kio_nepomuksearch.moc"
